from standards import standard_arrays
import numpy as np

def sensitivity(variables, constants,Kd_BlcR_GHB=700*10**-9):
    
    """Calculates the sensitivity of the sensor. To do this it calculates the 
    maximum amount of GHB for which 95% of DNA is still bound to BlcR. It also 
    calculates the minimum amount of GHB for which 5% of DNA is still bound 
    to BlcR. 
    
    Parameters:
    ----------
    variables: list
        A list containing the variables: log of mininimum of BlcR,
        log of maximum of BlcR, log of minimum of GHB, log of maximum of GHB, 
        log of minimum Kd BlcR and DNA, log of maximum Kd BlcR and DNA,
        log of minimum Kd BlcR and GHB, log of maximum Kd BlcR and GHB.
    constants: list
         A list containg the constants: Surface of electrode,
        Volume between the electrodes, Hill coefficient, Kd of BlcR to DNA,
        Kd of BlcR to GHB, Kd of BlcR to DNA when GHB is bound,
        Max surface coverage with DNA, DNA total, 
        concentration of DNA between electrodes,cutoff,GHB_drink, Max_BlcR.
    Kd_BlcR_GHB: float
        The Kd value between BlcR and GHB
    Returns
    ----------
    k1: list
        List containing the minimum GHB concentrations needed to unbind 95% of DNA.
    k2: list
        List containing the maximum GHB concentrations needed to still bind 95% of DNA.
    Kd_BlcR_DNA_0: list
        List containing the Kd values of BlcR to GHB that are used.
    """
    ## load constants
    Hill_coefficient=constants[2]
    Kd_BlcR_DNA=constants[3]
    ## load variables
    log_min_Ca=variables[0]
    log_max_Ca=variables[1]
    log_min_GHB=variables[2]
    log_max_GHB=variables[3]
    log_min_Kd_BlcR_DNA=variables[4]
    log_max_Kd_BlcR_DNA=variables[5]
    log_min_Kd_BlcR_GHB=variables[6]
    log_max_Kd_BlcR_GHB=variables[7]
    ## load arrays
    arrays=standard_arrays(log_min_Ca,log_max_Ca,log_min_GHB,log_max_GHB,log_min_Kd_BlcR_DNA,log_max_Kd_BlcR_DNA,log_min_Kd_BlcR_GHB,log_max_Kd_BlcR_GHB)
    Ca_0=arrays[2]
    C_GHB_00=arrays[3]
    Kd_BlcR_DNA_0=arrays[7]
    k1=[0]*len(Kd_BlcR_DNA_0)
    k2=[0]*len(Kd_BlcR_DNA_0)
    C_GHB_00=np.logspace(log_min_GHB,log_max_GHB,1000)
    ## two usefull constants to run the code
    a=0
    b=0  
    ## For a range of GHB concentrations we calculate the fraction bound when there is 10 
    ## times more BlcR than the Kd value between BlcR and DNA. we calculate the 
    ##maximum amount of GHB for which 95% of DNA is still bound to BlcR. we also 
    ##calculate the minimum amount of GHB for which 5% of DNA is still bound 
    ##to BlcR. We do this for a range of Kd's between BlcR and DNA.
    for j in range(len(Kd_BlcR_DNA_0)):
        Ca_0=Kd_BlcR_DNA_0[j]
        a=0 
        b=0
        for i in range(len(C_GHB_00)):
            C_GHB_0=C_GHB_00[i]
            
            Ca=Ca_0-(C_GHB_0+Ca_0+Kd_BlcR_GHB-(Ca_0**2-2*C_GHB_0*Ca_0+2*Ca_0*Kd_BlcR_GHB+C_GHB_0**2+Kd_BlcR_GHB**2+2*C_GHB_0*Kd_BlcR_GHB)**0.5)/2
            fb=1/(1+(Kd_BlcR_DNA/(Ca)**2)**Hill_coefficient)
            if (a==0):
                if (fb<0.01):
                    k1[j]=C_GHB_0
                    a=1
            if (b==0):
                if (fb<0.99):
                    k2[j]=C_GHB_0
                    b=1
    return Kd_BlcR_DNA_0,k1,k2
