import numpy as np

def find_range_BlcR(variables, constants,Kd_BlcR_DNA,Kd_BlcR_GHB):
    
    """Calculates the possible range of BlcR concentrations to still have a 99% 
    to 1% signal difference between two GHB concentrations. To do this it calculates the 
    maximum amount of GHB for which 99% of DNA is still bound to BlcR. It also 
    calculates the minimum amount of GHB for which 1% of DNA is still bound 
    to BlcR. Using these values the lower and upperbound BlcR concentrations
    are calculated that can be used with two given GHB concentrations 
    
    Parameters:
    ----------
    variables: list
        A list containing the variables: log of mininimum of BlcR,
        log of maximum of BlcR, log of minimum of GHB, log of maximum of GHB, 
        log of minimum Kd BlcR and DNA, log of maximum Kd BlcR and DNA,
        log of minimum Kd BlcR and GHB, log of maximum Kd BlcR and GHB.
    constants: list
        A list containg the constants: Surface of electrode,
        Volume between the electrodes, Hill coefficient, Kd of BlcR to DNA,
        Kd of BlcR to GHB, Kd of BlcR to DNA when GHB is bound,
        Max surface coverage with DNA, DNA total, 
        concentration of DNA between electrodes,cutoff,GHB_drink, Max_BlcR.
    Kd_BlcR_DNA: float
        The Kd value between BlcR and DNA
    Kd_BlcR_GHB: float
        The Kd value between BlcR and GHB
    Returns
    ----------
    Ca_00: list
        List containing the BlcR concentrations used.
    k1: list
        List containing the minimum GHB concentrations needed to unbind 99% of DNA.
    k2: list
        List containing the maximum GHB concentrations needed to still bind 99% of DNA.
    upperbound: float
        The highest possible BlcR concentration that can be used
    lowerbound: float
        The lowest possible BlcR concentration that can be used
    """
    ## load constants
    Hill_coefficient=constants[2]
    cutoff=constants[9]
    GHB_drink=constants[10]
    ## load variables
    log_min_Ca=variables[0]
    log_max_Ca=variables[1]
    log_min_GHB=variables[2]
    log_max_GHB=variables[3]
    ## load arrays
    C_GHB_00=np.logspace(log_min_GHB,log_max_GHB,100)
    Ca_00=np.logspace(log_min_Ca,log_max_Ca,1000)
    k1=[0]*(len(Ca_00))
    k2=[0]*(len(Ca_00))
    ## two usefull constants in the code  
    b=0
    a=0
    ## For a range of GHB concentrations we calculate the fraction bound when there is 10 
    ## times more BlcR than the Kd value between BlcR and DNA. we calculate the 
    ##maximum amount of GHB for which 99% of DNA is still bound to BlcR. we also 
    ##calculate the minimum amount of GHB for which 1% of DNA is still bound 
    ##to BlcR. We do this for a range BlcR concentrations. We also calculate the lower
    ## and upper bound concentrations of BlcR for which we can still achieve 99% to 1%
    ## difference between two GHB concentrations.
    for j in range(len(Ca_00)-1): 
        a=0
        b=0
        Ca_0=Ca_00[j]
        for i in range(len(C_GHB_00)):
            C_GHB_0=C_GHB_00[i]
            Ca=Ca_0-(C_GHB_0+Ca_0+Kd_BlcR_GHB-(Ca_0**2-2*C_GHB_0*Ca_0+2*Ca_0*Kd_BlcR_GHB+C_GHB_0**2+Kd_BlcR_GHB**2+2*C_GHB_0*Kd_BlcR_GHB)**0.5)/2
            fb=1/(1+(Kd_BlcR_DNA/(Ca)**2)**Hill_coefficient)
            if (a==0):
                if (fb<0.01):
                    k1[j]=C_GHB_0
                    a=1
            if (b==0):
                if (fb<0.99):
                   k2[j]=C_GHB_0
                   b=1
        if (k1[j]<GHB_drink and k1[j]>0):
            upperbound=Ca_0
        if (k2[j]<cutoff and k2[j]>0):
            lowerbound=Ca_0
    return Ca_00,k1,k2, upperbound, lowerbound

